

package jmetal.metaheuristics.IRA;

import jmetal.core.Algorithm;
import jmetal.core.Operator;
import jmetal.core.Problem;
import jmetal.core.Solution;
import jmetal.core.SolutionSet;
import jmetal.operators.crossover.CrossoverFactory;
import jmetal.operators.mutation.MutationFactory;
import jmetal.problems.ProblemFactory;
import jmetal.problems.DTLZ.DTLZ1;
import jmetal.problems.DTLZ.DTLZ2;
import jmetal.problems.DTLZ.DTLZ3;
import jmetal.problems.DTLZ.DTLZ4;
import jmetal.problems.DTLZ.DTLZ5;
import jmetal.problems.DTLZ.DTLZ6;
import jmetal.problems.DTLZ.DTLZ7;
import jmetal.problems.LZ09.*;
import jmetal.problems.WFG.WFG1;
import jmetal.problems.WFG.WFG2;
import jmetal.problems.WFG.WFG3;
import jmetal.problems.WFG.WFG4;
import jmetal.problems.WFG.WFG5;
import jmetal.problems.WFG.WFG6;
import jmetal.problems.WFG.WFG7;
import jmetal.problems.WFG.WFG8;
import jmetal.problems.WFG.WFG9;
import jmetal.problems.ZDT.ZDT1;
import jmetal.problems.ZDT.ZDT2;
import jmetal.problems.ZDT.ZDT3;
import jmetal.problems.ZDT.ZDT4;
import jmetal.problems.ZDT.ZDT6;
import jmetal.problems.cec2009Competition.*;
import jmetal.qualityIndicator.QualityIndicator;
import jmetal.qualityIndicator.fastHypervolume.wfg.Front;
import jmetal.qualityIndicator.fastHypervolume.wfg.WFGHV;
import jmetal.util.Configuration;
import jmetal.util.JMException;
import jmetal.util.comparators.DominanceComparator;
import jmetal.problems.M2M.*;

import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Comparator;
import java.util.HashMap;
import java.util.logging.FileHandler;
import java.util.logging.Logger;


public class MOEAD_IRA_main {
	public static Logger logger_; // Logger object
	public static FileHandler fileHandler_; // FileHandler object


	public static void printGD(String path,double[] GD){
	    try {
	      /* Open the file */
	      FileOutputStream fos   = new FileOutputStream(path)     ;
	      OutputStreamWriter osw = new OutputStreamWriter(fos)    ;
	      BufferedWriter bw      = new BufferedWriter(osw)        ;               
	      for (int i = 0; i < GD.length; i++) {  
	        bw.write(GD[i]+" ");
	        bw.newLine();        
	      }
	      
	      /* Close the file */
	      bw.close();
	    }catch (IOException e) {
	      Configuration.logger_.severe("Error acceding to the file");
	      e.printStackTrace();
	    }       
	  }
	
	public static void main(String[] args) throws JMException,
			SecurityException, IOException, ClassNotFoundException {

		
		for (int fun = 22; fun <= 40; fun++) {
			int runtimes = 1;
			double[] IGDarray = new double[runtimes];
			double[] Hypervolume = new double[runtimes];

			for (int i = 0; i < runtimes; i++) {
				Problem problem = null; // The problem to solve
				Algorithm algorithm; // The algorithm to use
				Operator crossover; // Crossover operator
				Operator mutation; // Mutation operator

				QualityIndicator indicators; // Object to get quality indicators

				HashMap parameters; // Operator parameters

				indicators = null;
				if (args.length == 1) {
					Object[] params = { "Real" };
					problem = (new ProblemFactory())
							.getProblem(args[0], params);
				} // if
				else if (args.length == 2) {
					Object[] params = { "Real" };
					problem = (new ProblemFactory())
							.getProblem(args[0], params);
					indicators = new QualityIndicator(problem, args[1]);
				} // if
				else {
					if (fun == 1) {
						problem = new ZDT1("Real");

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\ZDT1_501.txt");
					}
					if (fun == 2) {
						problem = new ZDT2("Real");

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\ZDT2_501.txt");
					}
					if (fun == 3) {
						problem = new ZDT3("Real");

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\ZDT3_269.txt");
					}
					if (fun == 4) {
						problem = new ZDT4("Real", 10);

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\ZDT4_501.txt");
					}
					if (fun == 5) {
						problem = new ZDT6("Real", 10);

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\ZDT6_774.txt");
					}
					if (fun == 6) {
						problem = new DTLZ1("Real", 10, 3);

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\DTLZ1.pf");
					}
					if (fun == 7) {
						problem = new DTLZ2("Real", 10, 3);

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\DTLZ2.pf");
					}
					if (fun == 8) {
						problem = new DTLZ3("Real", 10, 3);

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\DTLZ3.pf");
					}
					if (fun == 9) {
						problem = new DTLZ4("Real", 10, 3);

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\DTLZ4.pf");
					}
					if (fun == 10) {
						problem = new DTLZ5("Real", 10, 3);

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\DTLZ5.txt");
					}
					if (fun == 11) {
						problem = new DTLZ6("Real", 10, 3);

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\DTLZ6.txt");
					}
					if (fun == 12) {
						problem = new DTLZ7("Real", 10, 3);

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\DTLZ7.pf");
					}
					//------------------------------------------------------
					if (fun == 13) {
						problem = new WFG1("Real", 4, 8, 2);

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\WFG1_605.txt");
					}
					if (fun == 14) {
						problem = new WFG2("Real", 4, 8, 2);

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\WFG2_111.txt");
					}
					if (fun == 15) {
						problem = new WFG3("Real", 4, 8, 2);

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\WFG3_right.txt");
					}
					if (fun == 16) {
						problem = new WFG4("Real", 4, 8, 2);

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\WFG4_1181.txt");
					}
					if (fun == 17) {
						problem = new WFG5("Real", 4, 8, 2);

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\WFG5_694.txt");
					}
					if (fun == 18) {
						problem = new WFG6("Real", 4, 8, 2);

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\wfg6_166.txt");
					}
					if (fun == 19) {
						problem = new WFG7("Real", 4, 8, 2);

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\WFG7_2435.txt");
					}
					if (fun == 20) {
						problem = new WFG8("Real", 4, 8, 2);

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\WFG8_201.txt");
					}
					if (fun == 21) {
						problem = new WFG9("Real", 4, 8, 2);

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\WFG9_2591.txt");
					}
					
					if (fun == 22) {
						problem = new UF1("Real");

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\UF1.dat");
					}
					if (fun == 23) {
						problem = new UF2("Real");

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\UF2.dat");
					}
					if (fun == 24) {
						problem = new UF3("Real");

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\UF3.dat");
					}
					if (fun == 25) {
						problem = new UF4("Real");

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\UF4.dat");
					}
					if (fun == 26) {
						problem = new UF5("Real");

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\UF5.dat");
					}
					if (fun == 27) {
						problem = new UF6("Real");

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\UF6.dat");
					}
					if (fun == 28) {
						problem = new UF7("Real");

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\UF7.dat");
					}
					if (fun == 29) {
						problem = new UF8("Real");

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\UF8.DAT");
					}
					if (fun == 30) {
						problem = new UF9("Real");

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\UF9.DAT");
					}
					if (fun == 31) {
						problem = new UF10("Real");

						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\UF10.DAT");
					}
					
					//-----------------------------------------------------
					if(fun == 32){
						problem = new LZ09_F1("Real");
						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\LZ09_F1.pf");
					}
					if(fun == 33){
						problem = new LZ09_F2("Real");
						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\LZ09_F2.pf");
					}
					if(fun == 34){
						problem = new LZ09_F3("Real");
						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\LZ09_F3.pf");
					}
					if(fun == 35){
						problem = new LZ09_F4("Real");
						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\LZ09_F4.pf");
					}
					if(fun == 36){
						problem = new LZ09_F5("Real");
						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\LZ09_F5.pf");
					}
					if(fun == 37){
						problem = new LZ09_F6("Real");
						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\LZ09_F6.pf");
					}
					if(fun == 38){
						problem = new LZ09_F7("Real");
						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\LZ09_F7.pf");
					}
					if(fun == 39){
						problem = new LZ09_F8("Real");
						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\LZ09_F8.pf");
					}
					if(fun == 40){
						problem = new LZ09_F9("Real");
						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\LZ09_F9.pf");
					}
					
					//------------MOP------------------------------------
					if(fun == 41){
						problem = new MOP1("Real");
						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\pf_MOP1.dat");
					}
					if(fun == 42){
						problem = new MOP2("Real");
						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\pf_MOP2.dat");
					}
					if(fun == 43){
						problem = new MOP3("Real");
						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\pf_MOP3.dat");
					}
					if(fun == 44){
						problem = new MOP4("Real");
						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\pf_MOP4.dat");
					}
					if(fun == 45){
						problem = new MOP5("Real");
						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\pf_MOP5.dat");
					}
					if(fun == 46){
						problem = new MOP6("Real");
						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\pf_MOP6.dat");
					}
					if(fun == 47){
						problem = new MOP7("Real");
						indicators = new QualityIndicator(problem,
								"E:\\new_multiobjective\\jMetal\\Pareto_front\\pf_MOP7.dat");
					}
				} // else

				algorithm = new MOEAD_IRA(problem);
				
				Solution referencePoint = new Solution(problem);

				// Algorithm parameters
				if(fun >= 13 && fun <= 21){
					algorithm.setInputParameter("populationSize", 100);//WF1-9
					algorithm.setInputParameter("maxEvaluations", 25000);
					
					referencePoint.setObjective(0, 3.0);
					referencePoint.setObjective(1, 5.0);
				}
				if(fun >= 22 && fun <= 28){
					algorithm.setInputParameter("populationSize", 300);//UF1-7 for 2objectives
					algorithm.setInputParameter("maxEvaluations", 300000);
					
					referencePoint.setObjective(0, 2.0);
					referencePoint.setObjective(1, 2.0);
				}
				if(fun>=29 && fun <= 31){
					algorithm.setInputParameter("populationSize", 600);//UF8-10 for 3objectives
					algorithm.setInputParameter("maxEvaluations", 300000);
					
					referencePoint.setObjective(0, 2.0);
					referencePoint.setObjective(1, 2.0);
					referencePoint.setObjective(2, 2.0);
				}		
				if(fun >= 32 && fun <= 36){
					algorithm.setInputParameter("populationSize", 300);//F1-5 2objectives
					algorithm.setInputParameter("maxEvaluations", 150000);
					
					referencePoint.setObjective(0, 2.0);
					referencePoint.setObjective(1, 2.0);
				}
				if(fun == 37){
					algorithm.setInputParameter("populationSize", 600);//F6 3objectives
					algorithm.setInputParameter("maxEvaluations", 300000);
					
					referencePoint.setObjective(0, 2.0);
					referencePoint.setObjective(1, 2.0);
					referencePoint.setObjective(2, 2.0);
				}
				if(fun >= 38 && fun <= 40){
					algorithm.setInputParameter("populationSize", 300);//F7-9 2objectives
					algorithm.setInputParameter("maxEvaluations", 150000);
					
					referencePoint.setObjective(0, 2.0);
					referencePoint.setObjective(1, 2.0);
				}
				if(fun >= 41 && fun <= 45){//MOP1-MOP5
					algorithm.setInputParameter("populationSize", 100);//2objectives
					algorithm.setInputParameter("maxEvaluations", 300000);
					
					referencePoint.setObjective(0, 2.0);
					referencePoint.setObjective(1, 2.0);
				}
				if(fun >= 46 && fun <= 47){//MOP6-MOP7
					algorithm.setInputParameter("populationSize", 300);//3objectives
					algorithm.setInputParameter("maxEvaluations", 300000);
					
					referencePoint.setObjective(0, 2.0);
					referencePoint.setObjective(1, 2.0);
					referencePoint.setObjective(2, 2.0);
				}
				
				algorithm.setInputParameter("dataDirectory","E:\\new_multiobjective\\jMetal\\weight");

				// Crossover operator
				parameters = new HashMap();
				parameters.put("CR", 1.0);
				parameters.put("F", 0.5);
				crossover = CrossoverFactory.getCrossoverOperator(
						"DifferentialEvolutionCrossover", parameters);

				// Mutation operator
				parameters = new HashMap();
				parameters.put("probability",
						1.0 / problem.getNumberOfVariables());
				parameters.put("distributionIndex", 20.0);
				mutation = MutationFactory.getMutationOperator(
						"PolynomialMutation", parameters);

				algorithm.addOperator("crossover", crossover);
				algorithm.addOperator("mutation", mutation);

				// Execute the Algorithm
				SolutionSet population = algorithm.execute();
				 
				//population.printObjectivesToFile("IRAobj-Run" + i + "-Fun" + fun);
				IGDarray[i] = indicators.getCECIGD(population);
				
				SolutionSet s = new SolutionSet(population.size());
				Comparator c = new DominanceComparator();
				for(int j=0; j<population.size(); j++) {
					int v = c.compare(population.get(j), referencePoint);
					if(v == -1) {
						s.add(population.get(j));
					}
				}
				if(s.size() <= 2) {
					Hypervolume[i] = 0.0;
				}
				else {
					Front front = new Front(s.size(), referencePoint.getNumberOfObjectives(), s);
					WFGHV hv = new WFGHV(referencePoint.getNumberOfObjectives(), s.size(), referencePoint);
					Hypervolume[i] = hv.getHV(front);
				}
			}
			
			//printGD("IRAFUN" + fun + "IGD", IGDarray);
			//printGD("IRAFUN" + fun + "HV", Hypervolume);
			
			double sumHV = 0;
			double sumIGD = 0;
			for (int i = 0; i < runtimes; i++) {
				sumIGD += IGDarray[i];
				sumHV += Hypervolume[i];
			}
			System.out.println(sumIGD / runtimes + "\t" + sumHV / runtimes);
		}
	}
}
